/*
 *   kgoodstuff - a button bar a la FvwmButtons for KDE
 *   Copyright (C) 1998  Juergen Vigna
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>

#include <qfileinf.h>
#include <qmsgbox.h>

#include <ksimpleconfig.h>

#include "prefbutton.h"

#define XPOS  320
#define YPOS  10
#define WIDTH 65

#define Inherited PreferencesButtonData

extern bool isKdelnkFile(const char* filename);

PreferencesButton::PreferencesButton(QWidget* parent,const char* name)
	:Inherited( parent, name )
{
    lil = new KIconLoader;
    init(lil);
}

PreferencesButton::PreferencesButton(KIconLoader *il, QWidget* parent,
                                     const char* name)
	:Inherited( parent, name )
{
    lil = 0;
    init(il);
}

PreferencesButton::~PreferencesButton()
{
    if (lil)
        delete lil;
    delete ilb;
    delete kdndWidget;
}

void PreferencesButton::init(KIconLoader *il)
{
    ilb = new KIconLoaderButton(il, this);
    ilb->setGeometry(XPOS, YPOS, WIDTH, WIDTH);
    labelName->setAutoResize(true);
    labelName->setText(i18n("Name:"));
    setPos(labelName, buttonName);
    labelCmd->setAutoResize(true);
    labelCmd->setText(i18n("Command:"));
    setPos(labelCmd, buttonCommand);
    labelDesc->setAutoResize(true);
    labelDesc->setText(i18n("Description:"));
    setPos(labelDesc, buttonDesc);
    labelUseRows->setAutoResize(true);
    labelUseRows->setText(i18n("Use Rows:"));
    setPos(labelUseRows, buttonUseRows);
    labelUseCols->setAutoResize(true);
    labelUseCols->setText(i18n("Use Columns:"));
    setPos(labelUseCols, buttonUseCols);
    childKGS->setAutoResize(true);
    childKGS->setText(i18n("Child Buttonbar"));
    childCloseOpt->setAutoResize(true);
    childCloseOpt->setText(i18n("Close Child after first click"));
    labelSwallowName->setAutoResize(true);
    labelSwallowName->setText(i18n("Window Title:"));
    setPos(labelSwallowName, buttonSwallowName);
    labelSwallowCmd->setAutoResize(true);
    labelSwallowCmd->setText(i18n("Swallow Command:"));
    setPos(labelSwallowCmd, buttonSwallowCmd);
    okButton->setText(i18n("&OK"));
    applyButton->setText(i18n("&Apply"));
    cancelButton->setText(i18n("&Cancel"));
    data = 0;
    pokButton = okButton;
    kdndWidget = new KDNDDropZone(this, DndURL);
    CHECK_PTR(kdndWidget);
    connect(kdndWidget, SIGNAL(dropAction(KDNDDropZone *)),
            SLOT(slotDropEvent(KDNDDropZone *)));
}

void PreferencesButton::setPrefs(const char *bName, const char *bPixmap,
                                 const char *bCommand, const char *bDesc,
                                 const char *bSwallowName,
                                 const char *bSwallowCmd,
                                 bool childFlag, bool childCOptFlag,
                                 int useRows, int useCols)
{
    QString
        str;

    buttonName->setText(bName);
    ilb->setIcon(bPixmap);
    buttonCommand->setText(bCommand);
    buttonDesc->setText(bDesc);
    buttonSwallowName->setText(bSwallowName);
    buttonSwallowCmd->setText(bSwallowCmd);
    childKGS->setChecked(childFlag);
    childCloseOpt->setChecked(childCOptFlag);
    str.sprintf("%d",useRows);
    buttonUseRows->setText(str.data());
    str.sprintf("%d",useCols);
    buttonUseCols->setText(str.data());
}

const char *PreferencesButton::getButtonName()
{
    return buttonName->text();
}

const char *PreferencesButton::getButtonPixmap()
{
    return ilb->icon();
}

const char *PreferencesButton::getButtonCommand()
{
    return buttonCommand->text();
}

const char *PreferencesButton::getButtonDesc()
{
    return buttonDesc->text();
}

const char *PreferencesButton::getButtonSwallowName()
{
    return buttonSwallowName->text();
}

const char *PreferencesButton::getButtonSwallowCmd()
{
    return buttonSwallowCmd->text();
}

bool PreferencesButton::getButtonChildKGS()
{
    return childKGS->isChecked();
}

bool PreferencesButton::getButtonChildCloseOpt()
{
    return childCloseOpt->isChecked();
}

int PreferencesButton::getButtonUseRows()
{
    int i=atoi(buttonUseRows->text());
    if (i<=0) {
        buttonUseRows->setText("1");
        i=1;
    }
    return i;
}

int PreferencesButton::getButtonUseCols()
{
    int i=atoi(buttonUseCols->text());
    if (i<=0) {
        buttonUseCols->setText("1");
        i=1;
    }
    return i;
}

void PreferencesButton::setPos(QLabel *lab, QLineEdit *le)
{
    int pos, diff;

    pos = lab->x() + lab->width() + 5;
    diff = pos - le->x();
    if (diff > 0)
        le->setGeometry(pos, le->y(), le->width() - diff, le->height());
}

void PreferencesButton::okButtonCB()
{
    hide();
    emit okButtonPressed();
}

void PreferencesButton::applyButtonCB()
{
    emit okButtonPressed();
}

void PreferencesButton::slotDropEvent(KDNDDropZone * dz)
{
    QStrList &list = dz->getURLList();
    QString
        path,
        s = list.first();

    path = KURL(s).path();
    if (!KURL(s).isLocalFile() || !QFileInfo(path).exists()) {
        QMessageBox::warning(0,i18n("Error cannot find file:"),path,
                             i18n("Ok"));
        return;
    }
    if (!isKdelnkFile(path)) {
        QMessageBox::warning(0,i18n("Error:"),i18n("Not valid .kdelnk file"),
                             i18n("Ok"));
        return;
    }
    KSimpleConfig kconfig(path, true);
    kconfig.setGroup("KDE Desktop Entry");
    buttonName->setText(kconfig.readEntry("Name"));
    ilb->setIcon(kconfig.readEntry("Icon"));
    buttonCommand->setText(kconfig.readEntry("Exec"));
    buttonDesc->setText(kconfig.readEntry("Comment"));
}
